@@ -20,12 +20,13 @@
 
 #include "grib_api.h"
 
-void usage(char* prog) {
+void usage(const char* prog) {
   printf("Usage: %s grib_file grib_file ...\n",prog);
   exit(1);
 }
 
-int main(int argc, char** argv) {
+int main(int argc, char** argv)
+{
   int err = 0;
   long step=0;
   size_t nfiles;
