@@ -46,5 +46,8 @@ else
     GRIB_API_LIB=`pwd`/src
   fi
 
+  # Download the data needed for tests
+  ${data_dir}/download.sh "${data_dir}"
+
   set -u
 fi
