@@ -8,4 +8,4 @@
 # virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
 
 . ./include.sh
-${examples_dir}set_bitmap > /dev/null
+${examples_dir}f_set_bitmap > /dev/null
