@@ -945,21 +945,6 @@ int grib_c_new_from_samples(int* gid, char* name){
   return  GRIB_FILE_NOT_FOUND;
 }
 
-int grib_c_new_from_template(int* gid, char* name){
-  grib_handle *h = NULL;
-
-  h = grib_handle_new_from_template(NULL,name);
-  /*   grib_context_set_debug(h->context,1);*/
-
-  if(h){
-    push_handle(h,gid);
-    return GRIB_SUCCESS;
-  }
-
-  *gid = -1;
-  return  GRIB_INTERNAL_ERROR;
-}
-
 int grib_c_clone(int* gidsrc,int* giddest){
   grib_handle *src  = get_handle(*gidsrc);
   grib_handle *dest = NULL;
@@ -1046,7 +1031,12 @@ int grib_c_new_from_file(FILE* f, int* gid, int headers_only){
       return GRIB_SUCCESS;
     } else {
       *gid=-1;
-      return GRIB_END_OF_FILE;
+      if (err == GRIB_SUCCESS) {
+        return GRIB_END_OF_FILE;
+      } else {
+        /* A real error occurred */
+        return err;
+      }
     }
   }
 
