@@ -225,16 +225,16 @@ static grib_codetable* load_table(grib_accessor_codetable* self)
   if (*masterDir!=0) {
     sprintf(name,"%s/%s",masterDir,self->tablename);
     grib_recompose_name(h, NULL,name, recomposed,0);
-    filename=grib_context_full_path(c,recomposed);
+    filename=grib_context_full_defs_path(c,recomposed);
   } else {
     grib_recompose_name(h, NULL,self->tablename, recomposed,0);
-    filename=grib_context_full_path(c,recomposed);
+    filename=grib_context_full_defs_path(c,recomposed);
   }
 
   if (*localDir!=0) {
     sprintf(localName,"%s/%s",localDir,self->tablename);
     grib_recompose_name(h, NULL,localName, localRecomposed,0);
-    localFilename=grib_context_full_path(c,localRecomposed);
+    localFilename=grib_context_full_defs_path(c,localRecomposed);
   }
   
   next=c->codetable;
@@ -277,7 +277,7 @@ static int grib_load_codetable(grib_context* c,const char* filename,
   FILE *f = NULL;
   int lineNumber = 0;
 
-  grib_context_log(c,GRIB_LOG_DEBUG,"Loading code table form %s",filename);
+  grib_context_log(c,GRIB_LOG_DEBUG,"Loading code table from %s",filename);
 
   f=fopen(filename, "r");
   if (!f) return GRIB_IO_PROBLEM;
@@ -335,7 +335,7 @@ static int grib_load_codetable(grib_context* c,const char* filename,
 
     if(code <0 || code >= size)
     {
-      grib_context_log(c,GRIB_LOG_WARNING,"code_table_entry: invalide code in %s: %d (table size=%d)",filename,code,size);
+      grib_context_log(c,GRIB_LOG_WARNING,"code_table_entry: invalid code in %s: %d (table size=%d)",filename,code,size);
       continue;
     }
 
@@ -381,7 +381,6 @@ static int grib_load_codetable(grib_context* c,const char* filename,
     t->entries[code].title        = grib_context_strdup_persistent(c,title);
     t->entries[code].units        = grib_context_strdup_persistent(c,units);
 
-
   }
 
   fclose(f);
@@ -416,8 +415,6 @@ void grib_codetable_delete(grib_context* c) {
 
 }
 
-
-
 static void dump(grib_accessor* a, grib_dumper* dumper) {
   grib_accessor_codetable* self  = (grib_accessor_codetable*)a;
   char comment[2048];
@@ -629,7 +626,6 @@ static void destroy(grib_context* context,grib_accessor* a)
 		grib_context_free(context, a->vvalue);
 		a->vvalue=NULL;
 	}
-
 }
 
 static int  get_native_type(grib_accessor* a){
@@ -670,5 +666,3 @@ static int    unpack_long   (grib_accessor* a, long* val, size_t *len)
   *len = rlen;
   return GRIB_SUCCESS;
 }
-
-
