@@ -148,9 +148,9 @@ static long number_of_bits(unsigned long x) {
     const int count = sizeof(nbits)/sizeof(nbits[0]);
     long i=0;
     while (x>=*n) {
-        n++;
-        i++;
-        Assert(i<count);
+       n++;
+       i++;
+       Assert(i<count);
     }
     return i;
 }
@@ -222,7 +222,8 @@ static int  unpack_long(grib_accessor* a, long* val, size_t *len)
   b=grib_power(-binaryScaleFactor,2);
 
   /* self->bitsPerValue=(long)ceil(log((double)((max-min)*d+1))/log(2.0))-binaryScaleFactor; */
-  self->bitsPerValue=number_of_bits((unsigned long)(fabs(max-min)*b*d));
+  /* See GRIB-540 for why we use ceil */
+  self->bitsPerValue=number_of_bits( (unsigned long)ceil((fabs(max-min)*b*d)) );
   *val=self->bitsPerValue;
 
   grib_context_free(a->parent->h->context,values);
