@@ -29,6 +29,13 @@ extern "C" {
 #include <sys/time.h>
 #include <math.h>
 
+#if defined( __GNUC__) || defined(__clang__)
+#define DEPRECATED __attribute__((deprecated))
+#else
+#define DEPRECATED
+#endif
+
+
 #define GRIB_API_VERSION (GRIB_API_MAJOR_VERSION*10000+GRIB_API_MINOR_VERSION*100+GRIB_API_REVISION_VERSION)
 
 /* sections */
@@ -462,12 +469,13 @@ grib_handle* grib_handle_new_from_message_copy(grib_context* c, const void* data
 /**
 *  Create a handle from a read_only template resource.
 *  The message is copied at the creation of the handle
+*  This function is deprecated: please use grib_handle_new_from_samples
 *
 * @param c           : the context from which the handle will be created (NULL for default context)
 * @param res_name    : the resource name
 * @return            the new handle, NULL if the resource is invalid or a problem is encountered
 */
-grib_handle* grib_handle_new_from_template (grib_context* c, const char* res_name)  ;
+DEPRECATED grib_handle* grib_handle_new_from_template (grib_context* c, const char* res_name)  ;
 
 /**
  *  Create a handle from a message contained in a samples directory.
@@ -514,7 +522,7 @@ grib_multi_handle* grib_multi_handle_new     (grib_context* c);
  *  memory leaks.
  *
  * @param h           : The handle from which the sections are copied.
- * @param start_section : section number. Starting from this section all the sections to then end of the message will be copied.
+ * @param start_section : section number. Starting from this section all the sections to the end of the message will be copied.
  * @param mh           : The multi field handle on which the sections are appended.
  * @return            0 if OK, integer value on error
  */
@@ -548,7 +556,7 @@ int grib_multi_handle_write(grib_multi_handle* mh,FILE* f);
 *
 * @param h              : the grib handle to which the buffer should be gathered
 * @param message        : the pointer to be set to the handle's data
-* @param message_length : at exist, the message size in number of bytes
+* @param message_length : On exit, the message size in number of bytes
 * @return            0 if OK, integer value on error
 */
 int grib_get_message(grib_handle* h ,const void** message, size_t *message_length  );
@@ -559,8 +567,8 @@ int grib_get_message(grib_handle* h ,const void** message, size_t *message_lengt
 *
 * @param h              : the grib handle to which the buffer should be returned
 * @param message        : the pointer to the data buffer to be filled
-* @param message_length : at entry, the size in number of bytes of the allocated empty message.
-*                         At exist, the actual message length in number of bytes
+* @param message_length : On entry, the size in number of bytes of the allocated empty message.
+*                         On exit, the actual message length in number of bytes
 * @return            0 if OK, integer value on error
 */
 int grib_get_message_copy(grib_handle* h ,  void* message,size_t *message_length );
@@ -579,6 +587,19 @@ int grib_get_message_copy(grib_handle* h ,  void* message,size_t *message_length
 */
 grib_iterator*      grib_iterator_new      (grib_handle*   h, unsigned long flags,int* error);
 
+/**
+* Get latitude/longitude and data values.
+* The Latitudes, longitudes and values arrays must be properly allocated by the caller.
+* Their required dimension can be obtained by getting the value of the integer key "numberOfPoints".
+*
+* @param h           : handle from which geography and data values are taken
+* @param lats        : returned array of latitudes
+* @param lons        : returned array of longitudes
+* @param values      : returned array of data values
+* @return            0 if OK, integer value on error
+*/
+int grib_get_data(grib_handle *h, double *lats, double *lons, double *values, size_t *size);
+
 /**
 * Get the next value from an iterator.
 *
@@ -1184,6 +1205,16 @@ void grib_multi_support_on(grib_context* c);
 * @param c            : the context to be modified
 */
 void grib_multi_support_off(grib_context* c);
+
+/**
+*  Reset file handle in multiple field support mode
+*
+* @param c            : the context to be modified
+* @param f            : the file pointer
+*/
+void grib_multi_support_reset_file(grib_context* c, FILE* f);
+
+char* grib_samples_path(const grib_context *c);
 /*! @} */
 
 /**
@@ -1230,7 +1261,6 @@ grib_keys_iterator* grib_keys_iterator_new(grib_handle* h,unsigned long filter_f
 int grib_keys_iterator_next(grib_keys_iterator *kiter);
 
 
-
 /*! get the key name from the iterator
 *  @param kiter         : valid grib_keys_iterator
 *  @return              key name
@@ -1249,8 +1279,13 @@ int grib_keys_iterator_delete( grib_keys_iterator* kiter);
 */
 int grib_keys_iterator_rewind(grib_keys_iterator* kiter);
 
-
 int grib_keys_iterator_set_flags(grib_keys_iterator *kiter,unsigned long flags);
+
+int grib_keys_iterator_get_long(grib_keys_iterator *kiter, long *v, size_t *len);
+int grib_keys_iterator_get_double(grib_keys_iterator *kiter, double *v, size_t *len);
+int grib_keys_iterator_get_string(grib_keys_iterator *kiter, char *v, size_t *len);
+int grib_keys_iterator_get_bytes(grib_keys_iterator *kiter, unsigned char *v, size_t *len);
+
 /* @} */
 
 void grib_update_sections_lengths(grib_handle* h);
@@ -1277,6 +1312,7 @@ grib_handle* grib_handle_new_from_partial_message(grib_context* c,void* data, si
 int grib_is_missing(grib_handle* h, const char* key, int* err);
 int grib_is_defined(grib_handle* h, const char* key);
 int grib_set_missing(grib_handle* h, const char* key);
+/* The truncation is the Gaussian number (or order) */
 int grib_get_gaussian_latitudes(long truncation,double* latitudes);
 
 int grib_julian_to_datetime(double jd, long *year, long *month, long *day, long *hour, long *minute, long *second);
