@@ -303,7 +303,10 @@ int grib_write_unsigned_long(FILE* fh,unsigned long val)
 
 int grib_write_string(FILE* fh,const char* s)
 {
-    size_t len = strlen(s);
+    size_t len = 0;
+    if (s == NULL)
+        return GRIB_IO_PROBLEM;
+    len = strlen(s);
     grib_write_uchar(fh,(unsigned char)len);
     if (fwrite(s,1,len,fh)<len)
         return GRIB_IO_PROBLEM;
@@ -489,7 +492,6 @@ grib_index* grib_index_new(grib_context* c,const char* key,int *err)
 
 static void grib_index_values_delete(grib_context* c,grib_string_list* values)
 {
-
     if (!values) return;
 
     grib_index_values_delete(c,values->next);
@@ -510,7 +512,6 @@ static void grib_index_key_delete(grib_context* c,grib_index_key* keys)
     grib_context_free(c,keys->name);
     grib_context_free(c,keys);
 
-    return;
 }
 
 static long values_count=0;
@@ -546,6 +547,7 @@ static int grib_write_key_values(FILE* fh,grib_string_list* values)
     if (err) return err;
 
     err=grib_write_string(fh,values->value);
+    if (err) return err;
 
     err=grib_write_key_values(fh,values->next);
     if (err) return err;
@@ -619,13 +621,13 @@ static void grib_field_delete(grib_context* c,grib_field* field)
 
     grib_field_delete(c,field->next);
 
-    grib_file_close(field->file->name,&err);
-    field->file=NULL;
+    if (field->file) {
+        grib_file_close(field->file->name,&err);
+        field->file=NULL;
+    }
 
     grib_context_free(c,field);
 
-    return;
-
 }
 
 static void grib_field_tree_delete(grib_context* c,grib_field_tree* tree)
@@ -641,7 +643,12 @@ static void grib_field_tree_delete(grib_context* c,grib_field_tree* tree)
 
     grib_context_free(c,tree);
 
-    return;
+}
+
+static void grib_field_list_delete(grib_context* c, grib_field_list* field_list)
+{
+    if (!field_list) return;
+    grib_context_free(c,field_list);
 }
 
 void grib_index_delete(grib_index* index)
@@ -649,6 +656,7 @@ void grib_index_delete(grib_index* index)
     grib_file* file=index->files;
     grib_index_key_delete(index->context,index->keys);
     grib_field_tree_delete(index->context,index->fields);
+    grib_field_list_delete(index->context,index->fieldset);
     while (file) {
         grib_file* f=file;
         file=file->next;
@@ -734,7 +742,6 @@ int grib_index_write(grib_index* index,const char* filename)
         return err;
     }
 
-
     err=grib_write_index_keys(fh,index->keys);
     if (err) {
         grib_context_log(index->context,(GRIB_LOG_ERROR)|(GRIB_LOG_PERROR),
@@ -769,7 +776,7 @@ grib_index* grib_index_read(grib_context* c, const char* filename, int *err)
     unsigned char marker=0;
     char* identifier=NULL;
     int max=0;
-    FILE* fh;
+    FILE* fh = NULL;
 
     if (!c) c=grib_context_get_default();
 
@@ -784,12 +791,22 @@ grib_index* grib_index_read(grib_context* c, const char* filename, int *err)
     }
 
     identifier=grib_read_string(c,fh,err);
-    if (!identifier) return NULL;
+    if (!identifier) {
+        fclose(fh);
+        return NULL;
+    }
     grib_context_free(c,identifier);
 
     *err = grib_read_uchar(fh,&marker);
-    if(marker == NULL_MARKER) return NULL;
-    if(marker != NOT_NULL_MARKER) {*err=GRIB_CORRUPTED_INDEX;return NULL;}
+    if(marker == NULL_MARKER) {
+        fclose(fh);
+        return NULL;
+    }
+    if(marker != NOT_NULL_MARKER) {
+        *err=GRIB_CORRUPTED_INDEX;
+        fclose(fh);
+        return NULL;
+    }
 
     file = grib_read_files(c,fh,err);
     if (*err) return NULL;
@@ -806,7 +823,7 @@ grib_index* grib_index_read(grib_context* c, const char* filename, int *err)
     while (f) {
         grib_file_open(f->name,"r",err);
         if (*err) return NULL;
-        files[f->id]=grib_get_file(f->name,err);
+        files[f->id]=grib_get_file(f->name,err); /* fetch from pool */
         f=f->next;
     }
 
@@ -1082,6 +1099,8 @@ int grib_index_get_string(grib_index* index, const char* key, char** values, siz
     if (k->values_count>*size) return GRIB_ARRAY_TOO_SMALL;
     kv=k->values;
     while (kv) {
+        if (kv->value == NULL)
+            return GRIB_IO_PROBLEM;
         values[i++]=grib_context_strdup(index->context,kv->value);
         kv=kv->next;
     }
@@ -1099,8 +1118,7 @@ int grib_index_get_long(grib_index* index, const char* key, long* values, size_t
     while (k && strcmp(k->name,key)) k=k->next;
     if (!k) return GRIB_NOT_FOUND;
     if (k->type != GRIB_TYPE_LONG) {
-        grib_context_log(index->context,GRIB_LOG_ERROR,
-                "unable to get index %s as long");
+        grib_context_log(index->context,GRIB_LOG_ERROR, "unable to get index %s as long", key);
         return GRIB_WRONG_TYPE;
     }
     if (k->values_count > *size) return GRIB_ARRAY_TOO_SMALL;
@@ -1126,8 +1144,7 @@ int grib_index_get_double(grib_index* index,const char* key, double* values,size
     while (k && strcmp(k->name,key)) k=k->next;
     if (!k) return GRIB_NOT_FOUND;
     if (k->type != GRIB_TYPE_DOUBLE) {
-        grib_context_log(index->context,GRIB_LOG_ERROR,
-                "unable to get index %s as double");
+        grib_context_log(index->context,GRIB_LOG_ERROR, "unable to get index %s as double", key);
         return GRIB_WRONG_TYPE;
     }
     if (k->values_count>*size) return GRIB_ARRAY_TOO_SMALL;
@@ -1245,6 +1262,7 @@ int grib_index_select_string(grib_index* index,const char* skey,char* value)
 grib_handle* grib_index_get_handle(grib_field* field,int *err)
 {
     grib_handle* h=NULL;
+    Assert(field->file);
     grib_file_open(field->file->name,"r",err);
 
     if (*err!=GRIB_SUCCESS) return NULL;
@@ -1296,9 +1314,120 @@ static int grib_index_execute(grib_index* index)
     return 0;
 }
 
-void grib_index_dump(grib_index* index)
+static void grib_dump_key_values(FILE* fout, grib_string_list* values)
 {
+    grib_string_list* sl = values;
+    int first = 1; /* boolean for commas */
+    fprintf(fout, "values = ");
+    while (sl) {
+        if (!first) {
+            fprintf(fout, ", ");
+        }
+        fprintf(fout, "%s", sl->value);
+        first = 0;
+        sl = sl->next;
+    }
+    fprintf(fout, "\n");
+}
+static void grib_dump_index_keys(FILE* fout, grib_index_key* keys)
+{
+    if (!keys) return;
+    fprintf(fout, "key name = %s\n", keys->name);
+    /* fprintf(fout, "key type = %d\n", keys->type); */
 
+    grib_dump_key_values(fout, keys->values);
+    grib_dump_index_keys(fout, keys->next);
+}
+#if 0
+static void grib_dump_files(FILE* fout, grib_file* files)
+{
+    if (!files) return;
+    fprintf(fout, "file = %s\n", files->name);
+    fprintf(fout, "ID = %d\n", files->id);
+    grib_dump_files(fout, files->next);
+}
+static void grib_dump_field(FILE* fout, grib_field* field)
+{
+    if (!field) return;
+    fprintf(fout, "field name = %s\n", field->file->name);
+    /*fprintf(fout, "field FID = %d\n", field->file->id);
+    * fprintf(fout, "field offset = %ld\n", field->offset);
+    * fprintf(fout, "field length = %ld\n", field->length);
+    */
+
+    grib_dump_field(fout, field->next);
+}
+static void grib_dump_field_tree(FILE* fout, grib_field_tree* tree)
+{
+    if(!tree) return;
+    grib_dump_field(fout,tree->field);
+
+    fprintf(fout, "tree value = %s\n", tree->value);
+
+    grib_dump_field_tree(fout,tree->next_level);
+    grib_dump_field_tree(fout,tree->next);
+}
+#endif
+
+int grib_index_dump_file(FILE* fout, const char* filename)
+{
+    int err = 0;
+    grib_index* index = NULL;
+    grib_context* c = grib_context_get_default();
+    FILE* fh = NULL;
+
+    Assert(fout);
+    Assert(filename);
+    index = grib_index_read(c, filename, &err);
+    if (err) return err;
+    
+    /* To get the GRIB files referenced we have */
+    /* to resort to low level reading of the index file! */
+    fh=fopen(filename,"r");
+    if (fh) {
+        grib_file *file,*f;
+        char* identifier=NULL;
+        unsigned char marker=0;
+
+        identifier = grib_read_string(c,fh,&err);
+        if (err) return err;
+        grib_context_free(c,identifier);
+        err = grib_read_uchar(fh,&marker);
+        if (err) return err;
+        file = grib_read_files(c,fh,&err);
+        if (err) return err;
+        f=file;
+        while (f) {
+            fprintf(fout, "GRIB File: %s\n", f->name);
+            f=f->next;
+        }
+        fclose(fh);
+    }
+    
+    grib_index_dump(fout, index);
+    grib_index_delete(index);
+    
+    return GRIB_SUCCESS;
+}
+
+void grib_index_dump(FILE* fout, grib_index* index)
+{
+    if (!index) return;
+    Assert(fout);
+
+    /* The grib_dump_files does not print anything as  */
+    /* the index object does not store the file names! */
+    /* grib_dump_files(fout, index->files); */
+
+    fprintf(fout, "Index keys:\n");
+    grib_dump_index_keys(fout, index->keys);
+
+    /*
+    * fprintf(fout, "Index field tree:\n");
+    * grib_dump_field_tree(fout, index->fields);
+    */
+
+    fprintf(fout, "Index count = %d\n", index->count);
 }
 
 char* grib_get_field_file(grib_index* index,off_t *offset)
@@ -1341,7 +1470,7 @@ grib_handle* grib_handle_new_from_index(grib_index* index,int *err)
                 sizeof(grib_field_list));
         if (!index->fieldset) {
             grib_context_log(index->context,GRIB_LOG_ERROR,
-                    "unable to allocat %d bytes",
+                    "unable to allocate %d bytes",
                     sizeof(grib_field_list));
             return NULL;
         }
