@@ -15,6 +15,12 @@
 
 #include "grib_tools.h"
 #if HAVE_LIBJASPER
+/* Remove compiler warnings re macros being redefined */
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
 #include "jasper/jasper.h"
 #endif
 
@@ -97,30 +103,13 @@ grib_runtime_options options={
         0,         /* stop  */
         0,         /* mode  */
         0,         /* skip_all  */
-        {{0,},}    /* grib_values tolerance[MAX_KEYS] */
+        {{0,},},   /* grib_values tolerance[MAX_KEYS] */
+        0          /* infile_offset */
 
 	};
 
-static int is_index_file(const char* filename) {
-    FILE* fh;
-    char buf[8]={0,};
-    char* str="GRBIDX";
-    int ret=0;
-
-    fh=fopen(filename,"r");
-    if (!fh) return 0;
-
-    fread(buf,1,1,fh);
-    fread(buf,6,1,fh);
-
-    ret=!strcmp(buf,str);
-
-    fclose(fh);
-
-    return ret;
-}
-
-static grib_handle* grib_handle_new_from_file_x(grib_context* c,FILE* f,int mode,int *err) {
+static grib_handle* grib_handle_new_from_file_x(grib_context* c,FILE* f,int mode,int *err)
+{
     if (mode==MODE_HEADERS_ONLY)
         return grib_handle_headers_only_new_from_file(c,f,err);
     if (mode==MODE_GTS)
@@ -176,7 +165,8 @@ int grib_tool(int argc, char **argv)
 
 }
 
-static int grib_tool_with_orderby(grib_runtime_options* options) {
+static int grib_tool_with_orderby(grib_runtime_options* options)
+{
     int err=0;
     grib_failed *failed=NULL,*p=NULL;
     grib_handle* h=NULL;
@@ -257,7 +247,8 @@ static int grib_tool_with_orderby(grib_runtime_options* options) {
 
 char iobuf[1024*1024];
 
-static int grib_tool_without_orderby(grib_runtime_options* options) {
+static int grib_tool_without_orderby(grib_runtime_options* options)
+{
     int err=0;
     grib_failed *failed=NULL,*p=NULL;
     grib_handle* h=NULL;
@@ -282,6 +273,13 @@ static int grib_tool_without_orderby(grib_runtime_options* options) {
             perror(infile->name);
             exit(1);
         }
+        if (options->infile_offset) {
+            err=fseeko(infile->file, options->infile_offset, SEEK_SET);
+            if (err) {
+                fprintf(stderr, "Invalid file offset: %ld\n", options->infile_offset);
+                exit(1);
+            }
+        }
 
         setvbuf(infile->file,iobuf,_IOFBF,sizeof(iobuf));
 
@@ -291,7 +289,6 @@ static int grib_tool_without_orderby(grib_runtime_options* options) {
 
         grib_tool_new_file_action(options,infile);
 
-
         while(!options->skip_all && ((h = grib_handle_new_from_file_x(c,infile->file,options->mode,&err))
                 != NULL || err != GRIB_SUCCESS ) ) {
             infile->handle_count++;
@@ -354,7 +351,8 @@ static int grib_tool_without_orderby(grib_runtime_options* options) {
     return 0;
 }
 
-static int navigate(grib_field_tree* fields,grib_runtime_options* options) {
+static int navigate(grib_field_tree* fields,grib_runtime_options* options)
+{
     int err=0;
     if (!fields || options->stop) return 0;
 
@@ -381,7 +379,8 @@ static int navigate(grib_field_tree* fields,grib_runtime_options* options) {
     return err;
 }
 
-static int grib_tool_index(grib_runtime_options* options) {
+static int grib_tool_index(grib_runtime_options* options)
+{
     int err=0;
     grib_context* c=NULL;
     char* f1=options->infile->name;
@@ -473,7 +472,8 @@ static int grib_tool_index(grib_runtime_options* options) {
     return 0;
 }
 
-static int scan(grib_context* c,grib_runtime_options* options,const char* dir) {
+static int scan(grib_context* c,grib_runtime_options* options,const char* dir)
+{
     struct dirent *s;
     DIR *d;
     int err=0;
@@ -513,7 +513,8 @@ static int process(grib_context* c,grib_runtime_options* options,const char* pat
     return 0;
 }
 
-static int grib_tool_onlyfiles(grib_runtime_options* options) {
+static int grib_tool_onlyfiles(grib_runtime_options* options)
+{
     grib_context* c=NULL;
     grib_tools_file* infile=NULL;
 
@@ -532,8 +533,8 @@ static int grib_tool_onlyfiles(grib_runtime_options* options) {
     return 0;
 }
 
-static void grib_print_header(grib_runtime_options* options,grib_handle* h) {
-
+static void grib_print_header(grib_runtime_options* options,grib_handle* h)
+{
     size_t strlenkey=0;
     int width;
     int written_to_dump = 0; /* boolean */
@@ -577,8 +578,8 @@ static void grib_print_header(grib_runtime_options* options,grib_handle* h) {
     }
 }
 
-
-static void grib_tools_set_print_keys(grib_runtime_options* options, grib_handle* h, const char* ns) {
+static void grib_tools_set_print_keys(grib_runtime_options* options, grib_handle* h, const char* ns)
+{
     int i=0;
     grib_keys_iterator* kiter=NULL;
 
@@ -623,7 +624,8 @@ static void grib_tools_set_print_keys(grib_runtime_options* options, grib_handle
     }
 }
 
-static int to_skip(grib_handle* h,grib_values* v,int *err) {
+static int to_skip(grib_handle* h,grib_values* v,int *err)
+{
     double dvalue=0;
     int ret=0;
     long lvalue=0;
@@ -656,7 +658,8 @@ static int to_skip(grib_handle* h,grib_values* v,int *err) {
     return ret;
 }
 
-void grib_skip_check(grib_runtime_options* options,grib_handle* h) {
+void grib_skip_check(grib_runtime_options* options,grib_handle* h)
+{
     int i,ret=0;
     grib_values* v=NULL;
     for (i=0;i < options->constraints_count ;i++) {
@@ -735,7 +738,7 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h)
                 break;
             case GRIB_TYPE_LONG:
                 ret=grib_get_long( h,options->print_keys[i].name,&lvalue);
-                sprintf(value,"%d",(int)lvalue);
+                sprintf(value,"%ld", lvalue);
                 break;
             case GRIB_TYPE_BYTES:
                 ret=grib_get_string( h,options->print_keys[i].name,value,&len);
@@ -791,7 +794,7 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h)
         double v=0;
         /*if (grib_get_double_element(h,"values",options->index,&v) != GRIB_SUCCESS) {*/
         if (1) {
-            size_t size;
+            size_t size, the_index = 0;
             double* values;
             int err=0;
 
@@ -803,6 +806,12 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h)
             }
             values=grib_context_malloc_clear(h->context,size*sizeof(double));
             grib_get_double_array(h,"values",values,&size);
+            the_index = options->index;
+            if (the_index >= size) {
+                fprintf(dump_file,"\n");
+                fprintf(stderr, "invalid index value %d (should be between 0 and %ld)\n", options->index, size-1);
+                exit(1);
+            }
             v=values[options->index];
             grib_context_free(h->context,values);
         }
@@ -820,8 +829,8 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h)
     }
 }
 
-
-void grib_print_file_statistics(grib_runtime_options* options,grib_tools_file* file) {
+void grib_print_file_statistics(grib_runtime_options* options,grib_tools_file* file)
+{
     grib_failed* failed=NULL;
 
     Assert(file);
@@ -858,14 +867,16 @@ void grib_print_full_statistics(grib_runtime_options* options)
 
 void grib_tools_write_message(grib_runtime_options* options, grib_handle* h)
 {
-    const void *buffer; size_t size;
+    const void *buffer;
+    size_t size;
     grib_file* of=NULL;
     int err=0;
     int ioerr=0;
     char filename[1024]={0,};
     Assert(options->outfile!=NULL && options->outfile->name!=NULL);
 
-    if (options->error == GRIB_WRONG_LENGTH) return;
+    if (options->error == GRIB_WRONG_LENGTH)
+        return;
 
     if ((err=grib_get_message(h,&buffer,&size))!= GRIB_SUCCESS) {
         grib_context_log(h->context,GRIB_LOG_ERROR,"unable to get binary message\n");
